@@ -1,11 +1,17 @@
+import pathlib
+
 import pytest
 import kubernetes.client
+import yaml
 
 import kbatch_proxy.utils
 import kbatch_proxy.main
 import kbatch_proxy.patch
 
 
+HERE = pathlib.Path(__file__).parent
+
+
 @pytest.fixture
 def k8s_job() -> kubernetes.client.V1Job:
     job = kubernetes.client.models.V1Job(
@@ -177,22 +183,24 @@ def test_set_job_ttl_seconds_after_finished(k8s_job: kubernetes.client.V1Job):
 
 
 def test_add_node_affinity(k8s_job: kubernetes.client.V1Job):
-    kbatch_proxy.patch.patch(
-        k8s_job,
-        None,
-        username="foo",
-        job_node_affinity_required_label_key="hub.jupyter.org/node-purpose",
-        job_node_affinity_required_label_value="user",
-    )
+    job_template = yaml.safe_load((HERE / "job_template.yaml").read_text())
+    job_template = kbatch_proxy.utils.parse(
+        job_template, kubernetes.client.V1Job
+    ).to_dict()
 
-    node_affinity = k8s_job.spec.template.spec.affinity.node_affinity
-    terms = (
-        node_affinity.required_during_scheduling_ignored_during_execution.node_selector_terms[
-            0
-        ]
-        .node_selector_terms[0]
-        .match_expressions[0]
-    )
+    job_data = k8s_job.to_dict()
+
+    result = kbatch_proxy.utils.merge_json_objects(job_data, job_template)
+    result = kbatch_proxy.utils.parse(result, kubernetes.client.V1Job)
+
+    node_affinity = result.spec.template.spec.affinity.node_affinity
+    terms = node_affinity.required_during_scheduling_ignored_during_execution.node_selector_terms[
+        0
+    ].match_expressions[
+        0
+    ]
     assert terms.key == "hub.jupyter.org/node-purpose"
     assert terms.operator == "In"
     assert terms.values == ["user"]
+    assert job_data["spec"]["backoff_limit"] == 4  # overridden by template
+    assert result.spec.backoff_limit == 0  # overridden by template
